errorCodes={
    "ok": 0,
    "userTerm": 1,
    "invalidConfig": 1,
    'unknownError': 1024,
}

defaultChunkCount=100_000
excelBlankRowLimit=250
excelBlankColLimit=500
maxChunkSize = 650
SENTINEL = '!!!STOPQUEUE!!!'