from queue import Queue

import pytest

from piidigger.filehandlers import xls
from piidigger.queuefuncs import clearQ
from piidigger.logmanager import LogManager

@pytest.mark.filehandlers
@pytest.mark.parametrize('filename, expected_result', [
                            ('testdata/xls/empty-file.xls', [('', 0)]),
                            ('testdata/xls/random-data-table.xls', 
                                [
                                    ('First Name Last Name Gender Age Email Education Julia Montgomery Female 24 j.montgomery@randatmail.com Doctoral Amanda Bailey Female 30 a.bailey@randatmail.com Doctoral Lydia Brooks Female 24 l.brooks@randatmail.com Lower secondary Aldus Kelly Male 24 a.kelly@randatmail.com Bachelor John Thompson Male 30 j.thompson@randatmail.com Doctoral Ryan Douglas Male 24 r.douglas@randatmail.com Primary Savana Dixon Female 24 s.dixon@randatmail.com Lower secondary Ted Chapman Male 21 t.chapman@randatmail.com Primary Paul Andrews Male 26 p.andrews@randatmail.com Bachelor Albert Wright Male 30 a.wright@randatmail.com Master Martin Thomas Male 21 m.thomas@randatmail.com Upper secondary Tony Hunt Male 26 t.hunt@randatmail.com Lower secondary Adele Grant Female 30 a.grant@randatmail.com Upper secondary Mike Owens Male 27 m.owens@randatmail.com Primary Adison Higgins Male 29 a.higgins@randatmail.com Lower secondary James Cooper Male 18 j.cooper@randatmail.com Master Jessica Ross Female 24 j.ross@randatmail.com Upper secondary Abraham Andrews Male 23 a.andrews@randatmail.com Lower secondary Albert Campbell Male 20 a.campbell@randatmail.com Master Julian Alexander Male 20 j.alexander@randatmail.com Bachelor Aiden Smith Male 24 a.smith@randatmail.com Lower secondary Sam Alexander Male 21 s.alexander@randatmail.com', 1314),
                                    ('Lower secondary Chelsea Craig Female 20 c.craig@randatmail.com Upper secondary Sofia Cooper Female 26 s.cooper@randatmail.com Master Carl Nelson Male 19 c.nelson@randatmail.com Doctoral Annabella Cameron Female 30 a.cameron@randatmail.com Bachelor Sabrina Douglas Female 19 s.douglas@randatmail.com Doctoral Martin Cooper Male 28 m.cooper@randatmail.com Lower secondary Madaline Walker Female 22 m.walker@randatmail.com Doctoral Albert Stewart Male 28 a.stewart@randatmail.com Upper secondary Richard Ellis Male 21 r.ellis@randatmail.com Primary Albert Riley Male 18 a.riley@randatmail.com Lower secondary Isabella Moore Female 24 i.moore@randatmail.com Master Emily Sullivan Female 18 e.sullivan@randatmail.com Doctoral Adelaide Cameron Female 25 a.cameron@randatmail.com Upper secondary Sawyer Foster Male 29 s.foster@randatmail.com Master Blake Nelson Male 26 b.nelson@randatmail.com Doctoral Kristian Smith Male 20 k.smith@randatmail.com Master Sophia Ross Female 29 s.ross@randatmail.com Primary John Hawkins Male 24 j.hawkins@randatmail.com Master Justin Adams Male 29 j.adams@randatmail.com Lower secondary Victoria Richardson Female 18 v.richardson@randatmail.com Master George Gibson Male 27 g.gibson@randatmail.com Upper secondary Ned Perry Male 21 n.perry@randatmail.com Bachelor Amber Murray', 1303),
                                    ('Female 25 a.murray@randatmail.com Primary Edith Anderson Female 20 e.anderson@randatmail.com Primary Jordan Henderson Male 25 j.henderson@randatmail.com Upper secondary Lucas Harrison Male 26 l.harrison@randatmail.com Primary Sydney Riley Female 25 s.riley@randatmail.com Bachelor Dainton Barnes Male 30 d.barnes@randatmail.com Primary Tara Baker Female 26 t.baker@randatmail.com Doctoral Oscar Sullivan Male 25 o.sullivan@randatmail.com Primary Freddie Baker Male 20 f.baker@randatmail.com Master Alan Morris Male 28 a.morris@randatmail.com Upper secondary Vincent Barnes Male 21 v.barnes@randatmail.com Upper secondary Alisa Craig Female 23 a.craig@randatmail.com Upper secondary Adrianna Anderson Female 24 a.anderson@randatmail.com Doctoral Jordan Martin Male 23 j.martin@randatmail.com Lower secondary Garry Hill Male 19 g.hill@randatmail.com Primary Alisa Morris Female 21 a.morris@randatmail.com Bachelor Carl Barnes Male 24 c.barnes@randatmail.com Lower secondary Daryl Allen Male 21 d.allen@randatmail.com Master Aston Morris Male 21 a.morris@randatmail.com Doctoral Melanie Andrews Female 25 m.andrews@randatmail.com Upper secondary Elian Evans Male 28 e.evans@randatmail.com Primary Caroline Kelley Female 23 c.kelley@randatmail.com Doctoral Ryan Turner Male 25 r.turner@randatmail.com Doctoral', 1305),
                                    ('Andrew Ellis Male 29 a.ellis@randatmail.com Lower secondary Evelyn Kelly Female 24 e.kelly@randatmail.com Lower secondary Lyndon Andrews Male 25 l.andrews@randatmail.com Lower secondary Nicole Warren Female 20 n.warren@randatmail.com Lower secondary Roman Smith Male 19 r.smith@randatmail.com Lower secondary Savana Moore Female 28 s.moore@randatmail.com Upper secondary Sienna Wright Female 21 s.wright@randatmail.com Master Elise Murray Female 25 e.murray@randatmail.com Master Paige Russell Female 27 p.russell@randatmail.com Bachelor Adelaide Ferguson Female 25 a.ferguson@randatmail.com Upper secondary Myra Sullivan Female 25 m.sullivan@randatmail.com Lower secondary Dale Cunningham Male 19 d.cunningham@randatmail.com Doctoral Adison Armstrong Male 19 a.armstrong@randatmail.com Lower secondary Andrew Andrews Male 26 a.andrews@randatmail.com Lower secondary Tiana Riley Female 26 t.riley@randatmail.com Doctoral Amanda Taylor Female 19 a.taylor@randatmail.com Master Mike Campbell Male 23 m.campbell@randatmail.com Primary Elian Hall Male 27 e.hall@randatmail.com Primary Jordan Anderson Male 24 j.anderson@randatmail.com Bachelor Ned Holmes Male 20 n.holmes@randatmail.com Bachelor Sophia Hamilton Female 27 s.hamilton@randatmail.com Doctoral George Clark Male 23 g.clark@randatmail.com Bachelor', 1305),
                                    ('Darcy Evans Female 22 d.evans@randatmail.com Primary Thomas Nelson Male 30 t.nelson@randatmail.com Lower secondary Alissa Nelson Female 29 a.nelson@randatmail.com Lower secondary Maddie Barrett Female 20 m.barrett@randatmail.com Doctoral Ada Scott Female 30 a.scott@randatmail.com Primary Michael Baker Male 25 m.baker@randatmail.com Lower secondary Daisy Lloyd Female 26 d.lloyd@randatmail.com Master Kellan Gray Male 23 k.gray@randatmail.com Upper secondary Dominik Evans Male 27 d.evans@randatmail.com Primary Alan Alexander Male 22 a.alexander@randatmail.com Primary Ryan Warren Male 20 r.warren@randatmail.com Master Rafael Riley Male 21 r.riley@randatmail.com Master Catherine Morris Female 28 c.morris@randatmail.com Doctoral Darcy Gray Female 18 d.gray@randatmail.com Master Alina Morrison Female 23 a.morrison@randatmail.com Primary Walter Lloyd Male 24 w.lloyd@randatmail.com Lower secondary Alexia Johnson Female 19 a.johnson@randatmail.com Lower secondary Edith Wilson Female 30 e.wilson@randatmail.com Master Mary Craig Female 25 m.craig@randatmail.com Lower secondary Harold Howard Male 20 h.howard@randatmail.com Lower secondary Dexter Walker Male 25 d.walker@randatmail.com Bachelor Adrianna Grant Female 30 a.grant@randatmail.com Master Vanessa Mason Female 25 v.mason@randatmail.com', 1300),
                                    ('Master Brooke Campbell Female 26 b.campbell@randatmail.com Bachelor Adam Williams Male 23 a.williams@randatmail.com Lower secondary Camila Morris Female 22 c.morris@randatmail.com Upper secondary Carlos Johnston Male 26 c.johnston@randatmail.com Master Oscar Hunt Male 23 o.hunt@randatmail.com Lower secondary Melanie Williams Female 19 m.williams@randatmail.com Upper secondary Clark West Male 27 c.west@randatmail.com Bachelor Richard Barnes Male 28 r.barnes@randatmail.com Upper secondary Naomi Ryan Female 18 n.ryan@randatmail.com Doctoral Clark Harrison Male 26 c.harrison@randatmail.com Master Nicole Spencer Female 27 n.spencer@randatmail.com Upper secondary Alissa Henderson Female 23 a.henderson@randatmail.com Upper secondary Nicholas Bailey Male 30 n.bailey@randatmail.com Lower secondary Henry Stewart Male 18 h.stewart@randatmail.com Lower secondary Emily Williams Female 24 e.williams@randatmail.com Bachelor Charlotte Riley Female 28 c.riley@randatmail.com Bachelor Kristian Hill Male 23 k.hill@randatmail.com Upper secondary Honey Perry Female 27 h.perry@randatmail.com Lower secondary Frederick Hunt Male 20 f.hunt@randatmail.com Master George Harris Male 26 g.harris@randatmail.com Bachelor Rebecca Foster Female 23 r.foster@randatmail.com Doctoral Samantha Brooks Female 29 s.brooks@randatmail.com', 1316),
                                    ('Lower secondary Miranda Brown Female 30 m.brown@randatmail.com Upper secondary Cadie Clark Female 25 c.clark@randatmail.com Master Paige Wright Female 20 p.wright@randatmail.com Primary Reid Ferguson Male 20 r.ferguson@randatmail.com Lower secondary Eleanor Lloyd Female 23 e.lloyd@randatmail.com Master Adison Rogers Male 26 a.rogers@randatmail.com Doctoral Marcus Stevens Male 27 m.stevens@randatmail.com Bachelor Nicole Johnson Female 30 n.johnson@randatmail.com Doctoral Arnold Cameron Male 29 a.cameron@randatmail.com Master Catherine Baker Female 22 c.baker@randatmail.com Primary Alexander Richards Male 29 a.richards@randatmail.com Bachelor Daryl Miller Male 18 d.miller@randatmail.com Doctoral Victor Robinson Male 24 v.robinson@randatmail.com Doctoral Alfred Ferguson Male 24 a.ferguson@randatmail.com Primary Bruce Morgan Male 29 b.morgan@randatmail.com Bachelor Marcus West Male 18 m.west@randatmail.com Lower secondary Martin Foster Male 29 m.foster@randatmail.com Master Emma Lloyd Female 28 e.lloyd@randatmail.com Lower secondary Ashton Russell Male 21 a.russell@randatmail.com Master Brad Howard Male 23 b.howard@randatmail.com Upper secondary Blake Owens Male 24 b.owens@randatmail.com Doctoral Rubie Richardson Female 30 r.richardson@randatmail.com Lower secondary Ted Taylor Male 29', 1301),
                                    ('t.taylor@randatmail.com Master Paige Grant Female 26 p.grant@randatmail.com Lower secondary Sofia Stewart Female 24 s.stewart@randatmail.com Primary Abigail Henderson Female 29 a.henderson@randatmail.com Bachelor Brooke Reed Female 25 b.reed@randatmail.com Primary Brianna Ellis Female 29 b.ellis@randatmail.com Primary Chester Reed Male 25 c.reed@randatmail.com Lower secondary Emily Baker Female 25 e.baker@randatmail.com Primary Alisa Lloyd Female 28 a.lloyd@randatmail.com Bachelor Florrie Payne Female 21 f.payne@randatmail.com Master Sam Armstrong Male 25 s.armstrong@randatmail.com Upper secondary Edwin Hill Male 20 e.hill@randatmail.com Lower secondary Arianna Morris Female 26 a.morris@randatmail.com Bachelor Heather Nelson Female 29 h.nelson@randatmail.com Master Kelsey Turner Female 24 k.turner@randatmail.com Master Tyler Thomas Male 24 t.thomas@randatmail.com Upper secondary Connie Perkins Female 30 c.perkins@randatmail.com Bachelor Lyndon Murray Male 29 l.murray@randatmail.com Upper secondary Kimberly Harris Female 24 k.harris@randatmail.com Bachelor Abraham Kelley Male 22 a.kelley@randatmail.com Upper secondary Emma Warren Female 30 e.warren@randatmail.com Bachelor Lilianna Crawford Female 26 l.crawford@randatmail.com Bachelor Ted Watson Male 22 t.watson@randatmail.com Lower', 1301),
                                    ('secondary Fenton Foster Male 27 f.foster@randatmail.com Bachelor Cadie Cameron Female 21 c.cameron@randatmail.com Lower secondary Edgar Davis Male 20 e.davis@randatmail.com Lower secondary Mike Cooper Male 29 m.cooper@randatmail.com Bachelor Rebecca Andrews Female 29 r.andrews@randatmail.com Bachelor Amy Foster Female 19 a.foster@randatmail.com Lower secondary Annabella Martin Female 29 a.martin@randatmail.com Master Arianna Hamilton Female 30 a.hamilton@randatmail.com Lower secondary Miller Casey Male 22 m.casey@randatmail.com Master Gianna Hill Female 27 g.hill@randatmail.com Lower secondary Robert Cooper Male 20 r.cooper@randatmail.com Doctoral Penelope Morris Female 26 p.morris@randatmail.com Master Kristian Hawkins Male 21 k.hawkins@randatmail.com Upper secondary Spike Richardson Male 20 s.richardson@randatmail.com Master Adrianna Taylor Female 22 a.taylor@randatmail.com Bachelor John Stewart Male 30 j.stewart@randatmail.com Doctoral Rosie Warren Female 18 r.warren@randatmail.com Bachelor Alexander Jones Male 24 a.jones@randatmail.com Lower secondary Maria Higgins Female 29 m.higgins@randatmail.com Master Miller Smith Male 24 m.smith@randatmail.com Primary Garry Adams Male 29 g.adams@randatmail.com Upper secondary Martin Roberts Male 20 m.roberts@randatmail.com Primary Eleanor', 1302),
                                    ('Alexander Female 25 e.alexander@randatmail.com Master George Johnston Male 24 g.johnston@randatmail.com Upper secondary Sofia Nelson Female 21 s.nelson@randatmail.com Master Cherry Allen Female 18 c.allen@randatmail.com Master Jared Owens Male 24 j.owens@randatmail.com Doctoral Myra Douglas Female 26 m.douglas@randatmail.com Lower secondary Derek Stewart Male 30 d.stewart@randatmail.com Upper secondary Elise Cole Female 25 e.cole@randatmail.com Lower secondary Robert Ellis Male 19 r.ellis@randatmail.com Upper secondary Ned Johnson Male 28 n.johnson@randatmail.com Upper secondary Blake Elliott Male 22 b.elliott@randatmail.com Primary Samantha Stevens Female 22 s.stevens@randatmail.com Lower secondary Sienna Roberts Female 19 s.roberts@randatmail.com Doctoral Penelope Craig Female 19 p.craig@randatmail.com Upper secondary Tess Watson Female 19 t.watson@randatmail.com Bachelor Anna Hall Female 29 a.hall@randatmail.com Bachelor Daryl Richardson Male 18 d.richardson@randatmail.com Master Eddy Turner Male 20 e.turner@randatmail.com Doctoral Melissa Farrell Female 29 m.farrell@randatmail.com Doctoral Ada Craig Female 19 a.craig@randatmail.com Upper secondary Preston Thompson Male 28 p.thompson@randatmail.com Master Kirsten Barrett Female 21 k.barrett@randatmail.com Lower secondary Clark', 1300),
                                    ('Morrison Male 27 c.morrison@randatmail.com Bachelor Kristian Robinson Male 26 k.robinson@randatmail.com Lower secondary Mary Hill Female 25 m.hill@randatmail.com Upper secondary Roman Harper Male 19 r.harper@randatmail.com Primary James Morgan Male 27 j.morgan@randatmail.com Lower secondary Gianna Parker Female 18 g.parker@randatmail.com Master Frederick Morris Male 28 f.morris@randatmail.com Master Catherine Walker Female 20 c.walker@randatmail.com Doctoral Sawyer Walker Male 18 s.walker@randatmail.com Lower secondary Miley Cole Female 30 m.cole@randatmail.com Lower secondary John Fowler Male 25 j.fowler@randatmail.com Lower secondary Byron Martin Male 21 b.martin@randatmail.com Master Elise West Female 29 e.west@randatmail.com Bachelor Kristian Reed Male 23 k.reed@randatmail.com Bachelor Adelaide Hall Female 21 a.hall@randatmail.com Master Lilianna Howard Female 21 l.howard@randatmail.com Lower secondary Walter Hamilton Male 30 w.hamilton@randatmail.com Upper secondary Ellia Rogers Female 30 e.rogers@randatmail.com Bachelor Nicole Higgins Female 19 n.higgins@randatmail.com Doctoral Michelle Cameron Female 29 m.cameron@randatmail.com Lower secondary Joyce Morrison Female 29 j.morrison@randatmail.com Lower secondary Lilianna Thomas Female 18 l.thomas@randatmail.com Doctoral Marcus', 1301),
                                    ('Grant Male 21 m.grant@randatmail.com Master Dainton Allen Male 29 d.allen@randatmail.com Lower secondary Edwin Cunningham Male 30 e.cunningham@randatmail.com Primary Chloe Brooks Female 19 c.brooks@randatmail.com Primary Maddie Bailey Female 26 m.bailey@randatmail.com Upper secondary Sydney Hamilton Female 20 s.hamilton@randatmail.com Doctoral Maximilian Ryan Male 20 m.ryan@randatmail.com Lower secondary Nicole Adams Female 18 n.adams@randatmail.com Primary Luke Williams Male 30 l.williams@randatmail.com Upper secondary Amber Dixon Female 30 a.dixon@randatmail.com Upper secondary Mary Scott Female 30 m.scott@randatmail.com Bachelor Garry Murphy Male 29 g.murphy@randatmail.com Lower secondary Alfred Lloyd Male 29 a.lloyd@randatmail.com Master Reid Kelly Male 20 r.kelly@randatmail.com Doctoral Frederick Douglas Male 24 f.douglas@randatmail.com Lower secondary Stella Stevens Female 30 s.stevens@randatmail.com Lower secondary Sawyer Cooper Male 20 s.cooper@randatmail.com Master Briony Taylor Female 25 b.taylor@randatmail.com Bachelor Ryan Holmes Male 26 r.holmes@randatmail.com Primary Camila Perry Female 18 c.perry@randatmail.com Lower secondary Chelsea Brown Female 25 c.brown@randatmail.com Doctoral Paul Hunt Male 20 p.hunt@randatmail.com Primary Antony Morgan Male 28 a.morgan@randatmail.com', 1309),
                                    ('Primary Adam Murphy Male 29 a.murphy@randatmail.com Bachelor Roman Cooper Male 29 r.cooper@randatmail.com Primary Gianna Warren Female 19 g.warren@randatmail.com Master Brad Barnes Male 20 b.barnes@randatmail.com Primary Dexter Carter Male 18 d.carter@randatmail.com Doctoral Patrick Tucker Male 29 p.tucker@randatmail.com Master Sawyer Hawkins Male 24 s.hawkins@randatmail.com Master Alfred Payne Male 19 a.payne@randatmail.com Bachelor Aldus Wells Male 19 a.wells@randatmail.com Lower secondary Byron Wright Male 24 b.wright@randatmail.com Lower secondary Brad Wells Male 29 b.wells@randatmail.com Primary Fiona Elliott Female 25 f.elliott@randatmail.com Lower secondary Lilianna Morris Female 22 l.morris@randatmail.com Master Robert Scott Male 22 r.scott@randatmail.com Bachelor Antony Hill Male 30 a.hill@randatmail.com Bachelor Elise Davis Female 28 e.davis@randatmail.com Bachelor Maria Campbell Female 23 m.campbell@randatmail.com Primary Agata Taylor Female 21 a.taylor@randatmail.com Upper secondary Hailey Alexander Female 30 h.alexander@randatmail.com Master Honey Richardson Female 23 h.richardson@randatmail.com Lower secondary Abraham West Male 23 a.west@randatmail.com Upper secondary Chelsea Allen Female 30 c.allen@randatmail.com Upper secondary Stuart Cameron Male 25 s.cameron@randatmail.com', 1311),
                                    ('Master Lucas Clark Male 20 l.clark@randatmail.com Lower secondary Roman Harrison Male 19 r.harrison@randatmail.com Lower secondary Lucy Hall Female 18 l.hall@randatmail.com Bachelor Camila Morgan Female 23 c.morgan@randatmail.com Lower secondary Vincent Miller Male 25 v.miller@randatmail.com Doctoral Alexander Watson Male 24 a.watson@randatmail.com Bachelor Henry Sullivan Male 27 h.sullivan@randatmail.com Lower secondary Lucas Harper Male 23 l.harper@randatmail.com Upper secondary Lucy Lloyd Female 24 l.lloyd@randatmail.com Primary Amanda Baker Female 18 a.baker@randatmail.com Upper secondary Brad Richardson Male 25 b.richardson@randatmail.com Lower secondary Marcus Allen Male 30 m.allen@randatmail.com Bachelor Ada Wells Female 28 a.wells@randatmail.com Doctoral Aida Riley Female 28 a.riley@randatmail.com Upper secondary Elian Carter Male 30 e.carter@randatmail.com Master Daisy Hawkins Female 21 d.hawkins@randatmail.com Upper secondary Madaline Dixon Female 28 m.dixon@randatmail.com Doctoral Henry Davis Male 24 h.davis@randatmail.com Master Frederick Cameron Male 29 f.cameron@randatmail.com Bachelor Annabella Robinson Female 30 a.robinson@randatmail.com Upper secondary Kellan Harris Male 23 k.harris@randatmail.com Master Kevin Andrews Male 27 k.andrews@randatmail.com Doctoral Haris', 1302),
                                    ('Morgan Male 24 h.morgan@randatmail.com Master Daryl Spencer Male 29 d.spencer@randatmail.com Upper secondary Elian Rogers Male 18 e.rogers@randatmail.com Doctoral Miley Gray Female 23 m.gray@randatmail.com Lower secondary Maya Williams Female 26 m.williams@randatmail.com Master Alen Bailey Male 26 a.bailey@randatmail.com Primary Luke Davis Male 29 l.davis@randatmail.com Primary Edgar Cameron Male 24 e.cameron@randatmail.com Upper secondary Darcy Wilson Female 22 d.wilson@randatmail.com Upper secondary Reid Hall Male 23 r.hall@randatmail.com Primary Violet Alexander Female 27 v.alexander@randatmail.com Master Jasmine Henderson Female 18 j.henderson@randatmail.com Master Thomas Adams Male 23 t.adams@randatmail.com Master Paul Turner Male 22 p.turner@randatmail.com Master Cherry Richardson Female 20 c.richardson@randatmail.com Doctoral Rebecca Wilson Female 20 r.wilson@randatmail.com Doctoral Byron Stevens Male 18 b.stevens@randatmail.com Doctoral Jordan Kelly Male 27 j.kelly@randatmail.com Bachelor Clark Elliott Male 21 c.elliott@randatmail.com Upper secondary Bruce Turner Male 27 b.turner@randatmail.com Primary Vivian Morgan Female 18 v.morgan@randatmail.com Primary Lucia Richardson Female 19 l.richardson@randatmail.com Doctoral Violet Henderson Female 24 v.henderson@randatmail.com', 1301),
                                    ('Primary Violet Bailey Female 30 v.bailey@randatmail.com Doctoral Edgar Cooper Male 23 e.cooper@randatmail.com Doctoral Lily Hill Female 27 l.hill@randatmail.com Primary Valeria Johnson Female 21 v.johnson@randatmail.com Bachelor Emma Robinson Female 29 e.robinson@randatmail.com Primary Alford Spencer Male 22 a.spencer@randatmail.com Primary Blake Robinson Male 19 b.robinson@randatmail.com Lower secondary Emily Phillips Female 30 e.phillips@randatmail.com Primary Eddy Roberts Male 24 e.roberts@randatmail.com Doctoral Alina Allen Female 23 a.allen@randatmail.com Upper secondary Thomas Owens Male 28 t.owens@randatmail.com Bachelor Joyce Alexander Female 21 j.alexander@randatmail.com Bachelor Kristian Parker Male 24 k.parker@randatmail.com Primary Mary Perry Female 18 m.perry@randatmail.com Primary James Richards Male 24 j.richards@randatmail.com Bachelor Nicholas Roberts Male 20 n.roberts@randatmail.com Upper secondary Sofia Scott Female 19 s.scott@randatmail.com Upper secondary Alford Craig Male 29 a.craig@randatmail.com Bachelor Bruce Hawkins Male 25 b.hawkins@randatmail.com Lower secondary Daisy Myers Female 21 d.myers@randatmail.com Lower secondary Henry Cameron Male 21 h.cameron@randatmail.com Primary Martin Foster Male 20 m.foster@randatmail.com Lower secondary Freddie Dixon Male', 1303),
                                    ('23 f.dixon@randatmail.com Master Vanessa Grant Female 30 v.grant@randatmail.com Primary Alisa Lloyd Female 20 a.lloyd@randatmail.com Lower secondary Sawyer Kelly Male 24 s.kelly@randatmail.com Lower secondary Lily Phillips Female 29 l.phillips@randatmail.com Doctoral Tyler Lloyd Male 19 t.lloyd@randatmail.com Master Paul Spencer Male 27 p.spencer@randatmail.com Bachelor Savana Baker Female 22 s.baker@randatmail.com Upper secondary Tess Stevens Female 26 t.stevens@randatmail.com Lower secondary Samantha Owens Female 23 s.owens@randatmail.com Doctoral Byron Smith Male 19 b.smith@randatmail.com Doctoral Frederick Montgomery Male 25 f.montgomery@randatmail.com Bachelor Olivia Miller Female 27 o.miller@randatmail.com Bachelor Rubie Howard Female 29 r.howard@randatmail.com Lower secondary Adrianna Adams Female 20 a.adams@randatmail.com Lower secondary Oscar Andrews Male 20 o.andrews@randatmail.com Primary Adelaide Spencer Female 18 a.spencer@randatmail.com Master Connie Farrell Female 30 c.farrell@randatmail.com Upper secondary Emma Cole Female 26 e.cole@randatmail.com Bachelor Kirsten Riley Female 20 k.riley@randatmail.com Doctoral Maria Taylor Female 28 m.taylor@randatmail.com Master James Myers Male 29 j.myers@randatmail.com Bachelor Alfred Tucker Male 23 a.tucker@randatmail.com Upper', 1302),
                                    ('secondary Anna Bailey Female 28 a.bailey@randatmail.com Master Brooke Barrett Female 22 b.barrett@randatmail.com Doctoral Frederick Ryan Male 18 f.ryan@randatmail.com Bachelor Chelsea Ross Female 20 c.ross@randatmail.com Bachelor Bruce Scott Male 24 b.scott@randatmail.com Master Maximilian Clark Male 25 m.clark@randatmail.com Bachelor Ada Adams Female 24 a.adams@randatmail.com Doctoral Rosie Harper Female 19 r.harper@randatmail.com Primary Joyce Smith Female 25 j.smith@randatmail.com Upper secondary Wilson Hall Male 29 w.hall@randatmail.com Lower secondary Maya Baker Female 19 m.baker@randatmail.com Primary Honey Moore Female 21 h.moore@randatmail.com Upper secondary James Roberts Male 22 j.roberts@randatmail.com Primary Dale Barrett Male 24 d.barrett@randatmail.com Doctoral Sawyer Allen Male 25 s.allen@randatmail.com Bachelor Fenton Myers Male 28 f.myers@randatmail.com Bachelor Sarah Walker Female 27 s.walker@randatmail.com Doctoral Amanda Harper Female 19 a.harper@randatmail.com Master Eleanor Gray Female 19 e.gray@randatmail.com Doctoral Alina Ferguson Female 28 a.ferguson@randatmail.com Upper secondary Camila Davis Female 20 c.davis@randatmail.com Primary Anna Davis Female 27 a.davis@randatmail.com Bachelor Carlos Hawkins Male 26 c.hawkins@randatmail.com Bachelor Alfred Russell', 1302),
                                    ('Male 27 a.russell@randatmail.com Upper secondary Emily Watson Female 29 e.watson@randatmail.com Master Florrie Morrison Female 30 f.morrison@randatmail.com Lower secondary Rebecca Roberts Female 22 r.roberts@randatmail.com Doctoral Abraham Robinson Male 22 a.robinson@randatmail.com Doctoral Sawyer Miller Male 21 s.miller@randatmail.com Upper secondary Jessica Roberts Female 25 j.roberts@randatmail.com Lower secondary Edgar Kelley Male 18 e.kelley@randatmail.com Lower secondary Jordan Myers Male 29 j.myers@randatmail.com Primary Miranda Allen Female 27 m.allen@randatmail.com Bachelor Tyler Douglas Male 19 t.douglas@randatmail.com Primary Frederick Higgins Male 24 f.higgins@randatmail.com Lower secondary Deanna Perry Female 26 d.perry@randatmail.com Lower secondary Eric Russell Male 18 e.russell@randatmail.com Lower secondary Paige Stevens Female 24 p.stevens@randatmail.com Bachelor Preston Higgins Male 19 p.higgins@randatmail.com Lower secondary Brad Jones Male 26 b.jones@randatmail.com Upper secondary Violet Ellis Female 27 v.ellis@randatmail.com Bachelor Dexter Sullivan Male 29 d.sullivan@randatmail.com Doctoral Caroline Murray Female 28 c.murray@randatmail.com Upper secondary Rafael Russell Male 21 r.russell@randatmail.com Primary Anna Sullivan Female 21 a.sullivan@randatmail.com', 1302),
                                    ('Upper secondary Martin Douglas Male 29 m.douglas@randatmail.com Bachelor Chelsea Murray Female 23 c.murray@randatmail.com Primary Grace Robinson Female 30 g.robinson@randatmail.com Lower secondary Lydia Murphy Female 19 l.murphy@randatmail.com Upper secondary Fiona Reed Female 28 f.reed@randatmail.com Upper secondary Kirsten Spencer Female 21 k.spencer@randatmail.com Primary Roman Roberts Male 21 r.roberts@randatmail.com Doctoral April Andrews Female 25 a.andrews@randatmail.com Master Sofia Turner Female 28 s.turner@randatmail.com Lower secondary Sienna Ross Female 23 s.ross@randatmail.com Lower secondary Alissa Walker Female 30 a.walker@randatmail.com Lower secondary Honey Kelley Female 28 h.kelley@randatmail.com Doctoral Daisy Miller Female 29 d.miller@randatmail.com Bachelor Derek Phillips Male 26 d.phillips@randatmail.com Master Maximilian Myers Male 30 m.myers@randatmail.com Doctoral Stella Roberts Female 28 s.roberts@randatmail.com Lower secondary Arthur Owens Male 23 a.owens@randatmail.com Lower secondary Jacob Harris Male 30 j.harris@randatmail.com Upper secondary Reid Ryan Male 23 r.ryan@randatmail.com Master Valeria Cunningham Female 28 v.cunningham@randatmail.com Primary Stuart Reed Male 29 s.reed@randatmail.com Upper secondary Penelope Harris Female 27 p.harris@randatmail.com', 1308),
                                    ('Lower secondary Eddy Hall Male 23 e.hall@randatmail.com Bachelor Evelyn Hawkins Female 28 e.hawkins@randatmail.com Bachelor Miller Adams Male 25 m.adams@randatmail.com Primary Dale Rogers Male 26 d.rogers@randatmail.com Upper secondary Robert Elliott Male 30 r.elliott@randatmail.com Lower secondary Lilianna Mitchell Female 30 l.mitchell@randatmail.com Lower secondary Victoria Gibson Female 22 v.gibson@randatmail.com Bachelor Kristian Scott Male 25 k.scott@randatmail.com Lower secondary Amelia Anderson Female 27 a.anderson@randatmail.com Lower secondary Freddie Hill Male 29 f.hill@randatmail.com Upper secondary Chelsea Murphy Female 29 c.murphy@randatmail.com Primary Eddy Morrison Male 24 e.morrison@randatmail.com Doctoral Victoria Carroll Female 21 v.carroll@randatmail.com Bachelor Natalie Williams Female 29 n.williams@randatmail.com Primary Ned Elliott Male 30 n.elliott@randatmail.com Primary Reid Riley Male 20 r.riley@randatmail.com Lower secondary Adison Richardson Male 22 a.richardson@randatmail.com Lower secondary Sydney Evans Female 22 s.evans@randatmail.com Master Frederick Watson Male 21 f.watson@randatmail.com Primary David Harrison Male 28 d.harrison@randatmail.com Master Vanessa Cunningham Female 19 v.cunningham@randatmail.com Lower secondary Ellia Miller Female 20 e.miller@randatmail.com', 1320),
                                    ('Primary Lucas Allen Male 28 l.allen@randatmail.com Bachelor Maximilian Evans Male 24 m.evans@randatmail.com Doctoral Ned Harris Male 26 n.harris@randatmail.com Bachelor Carina Williams Female 23 c.williams@randatmail.com Doctoral Camila Gray Female 19 c.gray@randatmail.com Lower secondary Cadie Farrell Female 30 c.farrell@randatmail.com Lower secondary Dale Watson Male 23 d.watson@randatmail.com Bachelor Kevin Armstrong Male 22 k.armstrong@randatmail.com Doctoral Myra Ferguson Female 25 m.ferguson@randatmail.com Primary Kelsey Owens Female 28 k.owens@randatmail.com Lower secondary Annabella Jones Female 27 a.jones@randatmail.com Lower secondary Evelyn Richardson Female 26 e.richardson@randatmail.com Primary Andrew Hunt Male 27 a.hunt@randatmail.com Doctoral Brianna Allen Female 23 b.allen@randatmail.com Doctoral Nicole Henderson Female 28 n.henderson@randatmail.com Primary Fiona Crawford Female 22 f.crawford@randatmail.com Primary Jenna Scott Female 22 j.scott@randatmail.com Master Cherry Lloyd Female 27 c.lloyd@randatmail.com Primary Kelsey Moore Female 25 k.moore@randatmail.com Master Tara Ryan Female 23 t.ryan@randatmail.com Upper secondary Oscar Owens Male 28 o.owens@randatmail.com Primary Stuart Murphy Male 22 s.murphy@randatmail.com Lower secondary Tess Hawkins Female 24 t.hawkins@randatmail.com', 1322),
                                    ('Upper secondary Dainton Alexander Male 28 d.alexander@randatmail.com Master Kristian Barnes Male 23 k.barnes@randatmail.com Doctoral Amy Nelson Female 18 a.nelson@randatmail.com Lower secondary Lucas Barnes Male 25 l.barnes@randatmail.com Bachelor Agata Anderson Female 27 a.anderson@randatmail.com Primary Adam Allen Male 22 a.allen@randatmail.com Bachelor Naomi Bailey Female 23 n.bailey@randatmail.com Doctoral Charlotte Edwards Female 26 c.edwards@randatmail.com Lower secondary Eric Elliott Male 29 e.elliott@randatmail.com Lower secondary Sam Murphy Male 28 s.murphy@randatmail.com Lower secondary Samantha Murray Female 26 s.murray@randatmail.com Upper secondary Alan Fowler Male 29 a.fowler@randatmail.com Bachelor Maria Stevens Female 18 m.stevens@randatmail.com Master Edwin Anderson Male 25 e.anderson@randatmail.com Master Dale Parker Male 29 d.parker@randatmail.com Lower secondary Stuart Grant Male 28 s.grant@randatmail.com Master Heather Hunt Female 21 h.hunt@randatmail.com Upper secondary Preston Cooper Male 21 p.cooper@randatmail.com Master Belinda Brown Female 25 b.brown@randatmail.com Upper secondary Ryan Campbell Male 30 r.campbell@randatmail.com Lower secondary Lucia Carter Female 28 l.carter@randatmail.com Lower secondary Emma Mason Female 30 e.mason@randatmail.com Doctoral', 1303),
                                    ('Tyler Thompson Male 22 t.thompson@randatmail.com Doctoral Fiona Robinson Female 19 f.robinson@randatmail.com Upper secondary Catherine Douglas Female 19 c.douglas@randatmail.com Lower secondary Samantha Thomas Female 26 s.thomas@randatmail.com Doctoral Alberta Clark Female 19 a.clark@randatmail.com Lower secondary Kelvin Barrett Male 21 k.barrett@randatmail.com Bachelor Amanda Douglas Female 23 a.douglas@randatmail.com Doctoral Ryan Warren Male 27 r.warren@randatmail.com Primary Walter Higgins Male 21 w.higgins@randatmail.com Primary Tara Richards Female 30 t.richards@randatmail.com Doctoral Leonardo Hawkins Male 24 l.hawkins@randatmail.com Primary Aiden Johnston Male 29 a.johnston@randatmail.com Lower secondary Vivian Cooper Female 28 v.cooper@randatmail.com Primary Elise Montgomery Female 24 e.montgomery@randatmail.com Primary Ellia Taylor Female 23 e.taylor@randatmail.com Primary Clark Miller Male 30 c.miller@randatmail.com Bachelor Connie Hamilton Female 22 c.hamilton@randatmail.com Primary Ted Phillips Male 30 t.phillips@randatmail.com Doctoral Carlos Howard Male 21 c.howard@randatmail.com Bachelor Kirsten Thompson Female 27 k.thompson@randatmail.com Upper secondary Eric Andrews Male 29 e.andrews@randatmail.com Master Freddie Clark Male 29 f.clark@randatmail.com Doctoral Jasmine', 1304),
                                    ('Hawkins Female 18 j.hawkins@randatmail.com Lower secondary Fiona Stevens Female 22 f.stevens@randatmail.com Primary Paige Crawford Female 21 p.crawford@randatmail.com Doctoral Jacob Stevens Male 23 j.stevens@randatmail.com Bachelor Dominik Spencer Male 20 d.spencer@randatmail.com Doctoral Lily Thompson Female 22 l.thompson@randatmail.com Upper secondary Florrie Richardson Female 22 f.richardson@randatmail.com Doctoral Dainton Warren Male 26 d.warren@randatmail.com Lower secondary Jack Stevens Male 22 j.stevens@randatmail.com Master Lydia Stevens Female 25 l.stevens@randatmail.com Upper secondary Lana Cooper Female 27 l.cooper@randatmail.com Doctoral Heather Phillips Female 23 h.phillips@randatmail.com Master Brianna Carroll Female 27 b.carroll@randatmail.com Doctoral Rafael Hamilton Male 26 r.hamilton@randatmail.com Doctoral Kirsten Williams Female 30 k.williams@randatmail.com Bachelor Dale Smith Male 30 d.smith@randatmail.com Master Maria Clark Female 23 m.clark@randatmail.com Primary Savana Stewart Female 28 s.stewart@randatmail.com Master Leonardo Holmes Male 22 l.holmes@randatmail.com Master Leonardo Baker Male 23 l.baker@randatmail.com Doctoral Briony Hunt Female 28 b.hunt@randatmail.com Primary Anna Holmes Female 30 a.holmes@randatmail.com Primary Adam Ellis Male 18 a.ellis@randatmail.com', 1315),
                                    ('Lower secondary Adrian Harrison Male 21 a.harrison@randatmail.com Doctoral Walter Holmes Male 27 w.holmes@randatmail.com Primary Robert Wilson Male 25 r.wilson@randatmail.com Primary Alexia Russell Female 26 a.russell@randatmail.com Lower secondary Kirsten Nelson Female 24 k.nelson@randatmail.com Master Jacob Montgomery Male 22 j.montgomery@randatmail.com Bachelor Grace Nelson Female 30 g.nelson@randatmail.com Upper secondary Adrianna Evans Female 30 a.evans@randatmail.com Bachelor Alisa Clark Female 26 a.clark@randatmail.com Lower secondary Nicole Cooper Female 25 n.cooper@randatmail.com Primary Catherine Wells Female 30 c.wells@randatmail.com Master Paul Rogers Male 20 p.rogers@randatmail.com Lower secondary Haris Ryan Male 22 h.ryan@randatmail.com Bachelor Camila Harrison Female 22 c.harrison@randatmail.com Bachelor Belinda Richardson Female 20 b.richardson@randatmail.com Upper secondary Alexia Harper Female 28 a.harper@randatmail.com Upper secondary Vivian Owens Female 23 v.owens@randatmail.com Bachelor Roman Owens Male 23 r.owens@randatmail.com Master Alisa Richardson Female 18 a.richardson@randatmail.com Primary Valeria Carter Female 25 v.carter@randatmail.com Primary Tony Payne Male 29 t.payne@randatmail.com Doctoral Oliver Payne Male 19 o.payne@randatmail.com Lower secondary', 1303),
                                    ('Samantha Watson Female 23 s.watson@randatmail.com Upper secondary David Warren Male 21 d.warren@randatmail.com Bachelor Camila Fowler Female 25 c.fowler@randatmail.com Upper secondary Sawyer Richards Male 30 s.richards@randatmail.com Master Kristian Chapman Male 22 k.chapman@randatmail.com Primary Leonardo Payne Male 25 l.payne@randatmail.com Master Madaline Russell Female 19 m.russell@randatmail.com Primary Elian Cameron Male 21 e.cameron@randatmail.com Doctoral Dale Hunt Male 18 d.hunt@randatmail.com Primary Mike Evans Male 20 m.evans@randatmail.com Primary Eleanor Casey Female 26 e.casey@randatmail.com Doctoral John Thompson Male 22 j.thompson@randatmail.com Upper secondary Emily Taylor Female 23 e.taylor@randatmail.com Lower secondary Michael Davis Male 22 m.davis@randatmail.com Bachelor Lyndon Kelly Male 30 l.kelly@randatmail.com Bachelor Joyce Dixon Female 19 j.dixon@randatmail.com Bachelor Adison Myers Male 30 a.myers@randatmail.com Doctoral Brooke Harper Female 18 b.harper@randatmail.com Lower secondary', 1026),
                                ]),
                            ('testdata/xls/test-1sheet-1cell.xls', [('S1R1C1', 6)]),
                            ('testdata/xls/test-1sheet-1row.xls', [("S1R1C1 S1R1C2 S1R1C3 S1R1C4", 27)]),
                            ('testdata/xls/test-1sheet-10row-table.xls', 
                                [
                                    ('Sheet Row Column Text 1 1 3 S1R1C3 1 2 3 S1R2C3 1 3 3 S1R3C3 1 4 3 S1R4C3 1 5 3 S1R5C3 1 6 3 S1R6C3 1 7 3 S1R7C3 1 8 3 S1R8C3 1 9 3 S1R9C3 1 10 3 S1R10C3', 153),
                                ]),
                            ('testdata/xls/test-2sheet-10row-table.xls', 
                                [
                                    ('Sheet Row Column Text 1 1 3 S1R1C3 1 2 3 S1R2C3 1 3 3 S1R3C3 1 4 3 S1R4C3 1 5 3 S1R5C3 1 6 3 S1R6C3 1 7 3 S1R7C3 1 8 3 S1R8C3 1 9 3 S1R9C3 1 10 3 S1R10C3', 153),
                                    ('Sheet Row Column Text 1 1 3 S1R1C3 1 2 3 S1R2C3 1 3 3 S1R3C3 1 4 3 S1R4C3 1 5 3 S1R5C3 1 6 3 S1R6C3 1 7 3 S1R7C3 1 8 3 S1R8C3 1 9 3 S1R9C3 1 10 3 S1R10C3', 153),
                            ]),
                            ('testdata/xls/does-not-exist.xls', []),
                          ]
                  )
def test_read_xls_file(filename, expected_result):
    # Cut back from the default of 100,000 to make testing a bit easier on super-large files.  This is based on the premise that if it can handle 2 chunks (1300 bytes), it can handle 100,000 (61MB).
    maxChunkCount=2
    logQ = Queue()
    logManager=LogManager(logFile='test.log', logLevel='INFO', logQueue=Queue())

    result: list = []

    for content in xls.readFile(filename, logManager, maxChunkCount):
        result.append((content, len(content)))

    print(f'Result: "{result}"')
        

    clearQ(logQ)

    assert result == expected_result